'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();var _get = function get(_x, _x2, _x3) {var _again = true;_function: while (_again) {var object = _x, property = _x2, receiver = _x3;desc = parent = getter = undefined;_again = false;if (object === null) object = Function.prototype;var desc = Object.getOwnPropertyDescriptor(object, property);if (desc === undefined) {var parent = Object.getPrototypeOf(object);if (parent === null) {return undefined;} else {_x = parent;_x2 = property;_x3 = receiver;_again = true;continue _function;}} else if ('value' in desc) {return desc.value;} else {var getter = desc.get;if (getter === undefined) {return undefined;}return getter.call(receiver);}}};function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { 'default': obj };}function _defineProperty(obj, key, value) {if (key in obj) {Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true });} else {obj[key] = value;}return obj;}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}function _inherits(subClass, superClass) {if (typeof superClass !== 'function' && superClass !== null) {throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass);}subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } });if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;}var _react = require('react');var _react2 = _interopRequireDefault(_react);var _ReduxStoreJs = require(

'./ReduxStore.js');var _ReduxStoreJs2 = _interopRequireDefault(_ReduxStoreJs);var _helperJs = require(
'./helper.js');

/**
 * Components that wanted to use Soya's redux container may use this.
 * DataComponent encapsulates how, when, what, and where to fetch context.
 * Relationship between DataComponent and ReduxStore is as follows:
 *
 * 1) DataComponent may register Segments that it needs from ReduxStore.
 * 2) DataComponent may manually query pieces of Segments it has registered
 *    using each Segment's action creator object or ReduxStore.
 * 3) DataComponent may subscribe to a query. This automatically sets the
 *    query result to its React state. Each change in the query result will
 *    also trigger a setState() call to the subscribing DataComponent.
 *
 * DataComponent extends React's prop and state relationship:
 *
 * 1) Props are given by owner components to their ownee.
 * 2) Props is the prime determinator of state in components.
 * 3) Props is the prime determinator of query subscription in DataComponent.
 * 4) Query subscription determines part of DataComponent's state.
 *
 * @CLIENT_SERVER
 */var 
DataComponent = (function (_React$Component) {_inherits(DataComponent, _React$Component);










  function DataComponent(props, context) {_classCallCheck(this, DataComponent);
    _get(Object.getPrototypeOf(DataComponent.prototype), 'constructor', this).call(this, props, context);

    // Both actions and unsubscribe is not part of state because it shouldn't
    // affect rendering in any way. Actions are determined at mounting and
    // shouldn't change throughout this component's lifecycle. Unsubscribe
    // contains functions that unsubscribes this component from state changes,
    // but it's the actual state that matters.
    this.__soyaActions = {};
    this.__soyaUnsubscribe = {};

    var reduxStore = this.getReduxStore();
    var config = this.getConfig();
    if (!reduxStore || !reduxStore.__isReduxStore) {
      throw new Error('ReduxStore is not properly wired to this data component: ' + this.constructor + '.');}

    if (!config || typeof config != 'object') {
      throw new Error('Config object is not properly wired to this data component: ' + this.constructor + '.');}}



  /**
   * Defaults to getting ReduxStore instance from props.store. If you want to
   * use context you can override this method.
   *
   * @returns {ReduxStore}
   */_createClass(DataComponent, [{ key: 'getReduxStore', value: 
    function getReduxStore() {
      return this.props.reduxStore;}


    /**
     * Defaults to getting Config object from props.config. If you want to use
     * context you can override this method.
     *
     * @returns {Object}
     */ }, { key: 'getConfig', value: 
    function getConfig() {
      return this.props.config;}


    /**
     * This method is called at componentWilMount(). DataComponent will register
     * the returned Segment constructors to ReduxStore. Child components must
     * override this static method to specify their data requirements.
     *
     * Client-side or server-side config will be provided, depending on where
     * this DataComponent is instantiated.
     *
     * @return {Array<Class<Segment>>}
     */ }, { key: 'subscribeQueries', 




    /**
     * Subscribes to queries to a Segment already registered at
     * getSegmentDependencies().
     *
     * NOTE: Subscription must be done with subscribe() method. Method must also
     * use the given props instead of this.props.
     *
     * @param {Object} nextProps
     */value: 
    function subscribeQueries(nextProps) {}



    /**
     * @param {string} segmentId
     * @param {any} query
     * @param {string} stateName
     * @param {?Object} hydrationOption
     */ }, { key: 'subscribe', value: 
    function subscribe(segmentId, query, stateName, hydrationOption) {var _this = this;
      // Unsubscribe if already subscribed.
      this.unsubscribe(stateName);

      console.log('[DATA] Subscribe', this, segmentId);
      var callback = function callback(newState) {
        _this.setState(_defineProperty({}, stateName, newState));};


      var storeRef = this.getReduxStore().subscribe(
      segmentId, query, callback, this, hydrationOption);
      this.unsubscribe[segmentId] = storeRef.unsubscribe;
      this.setState(_defineProperty({}, 
      stateName, storeRef.getState()));}



    /**
     * @param {string} stateName
     */ }, { key: 'unsubscribe', value: 
    function unsubscribe(stateName) {
      if (this.__soyaUnsubscribe[stateName]) {
        console.log('[DATA] Un-subscribe', this, stateName);
        this.__soyaUnsubscribe[stateName]();
        delete this.__soyaUnsubscribe[stateName];}}



    /**
     * Returns action creator of an already registered segment.
     *
     * @param {string} segmentId
     * @return {Object}
     */ }, { key: 'getActionCreator', value: 
    function getActionCreator(segmentId) {
      if (!this.__soyaActions.hasOwnProperty(segmentId)) {
        throw new Error('Unable to get action creator for segment \'' + segmentId + '\', segment is not registered.');}

      return this.__soyaActions[segmentId];}


    /**
     * Semantics are very similar to shouldComponentUpdate(). This method
     * determines whether to re-run subscribeQueries() or not when this component
     * will receive new props.
     *
     * Props is the prime determinator of query subscription in DataComponent.
     *
     * Default implementation uses shallowEqual. User can override as needed.
     *
     * @param {Object} nextProps
     * @return {boolean}
     */ }, { key: 'shouldSubscriptionsUpdate', value: 
    function shouldSubscriptionsUpdate(nextProps) {
      var shouldUpdateSubscription = !(0, _helperJs.isEqualShallow)(this.props, nextProps);
      console.log('[DATA] Should update subscriptions', this, shouldUpdateSubscription);
      return shouldUpdateSubscription;}


    /**
     * @param {Class<Segment>} segmentClass
     */ }, { key: '_register', value: 
    function _register(segmentClass) {
      var actionCreator = this.getReduxStore().register(segmentClass);
      this.__soyaActions[segmentClass.id()] = actionCreator;}


    /**
     * This implementation assumes:
     *
     * 1) All DataComponent(s) render only using props and states (i.e. they
     *    are pure render components). Child classes can override this method
     *    if they are not.
     * 2) All DataComponent(s) parents send their props as immutable objects
     *    (i.e. if the prop changes, they recreate the whole object).
     * 3) All DataComponent(s) set their state as immutable objects
     *    (i.e. if state changes, they recreate the whole object).
     *
     * Based on the above assumptions, this method will do a shallow comparison of
     * both state and props. If the above assumptions are not correct, user can
     * override this method.
     *
     * @param {Object} nextProps
     * @param {Object} nextState
     * @return {boolean}
     */ }, { key: 'shouldComponentUpdate', value: 
    function shouldComponentUpdate(nextProps, nextState) {
      var shouldUpdate = !(0, _helperJs.isEqualShallow)(this.props, nextProps) || !(0, _helperJs.isEqualShallow)(this.state, nextState);
      console.log('[DATA] Should update?', this, shouldUpdate);
      return shouldUpdate;} }, { key: 'componentWillUpdate', value: 


    function componentWillUpdate(nextProps, nextState) {
      console.log('[DATA] Will update', this, nextProps, nextState);}


    /**
     * @param {Object} nextProps
     */ }, { key: 'componentWillReceiveProps', value: 
    function componentWillReceiveProps(nextProps) {
      // TODO: Logger at client! Remove if debug is set to false!
      if (this.shouldSubscriptionsUpdate(nextProps)) {
        // If query subscriptions update, we need to remove past subscriptions.
        // Otherwise this component may unnecessarily re-render each time the
        // already unrelated segment piece changes. Since this is costly, this
        // is why shouldSubscriptionsUpdate() is important.
        this.getReduxStore().unsubscribe(this);
        this.subscribeQueries(nextProps);}}



    /**
     * Registers the store. Run at both client and server side when rendering.
     */ }, { key: 'componentWillMount', value: 
    function componentWillMount() {
      console.log('[DATA] Mounting', this, this.props);
      var i, segmentClasses = this.constructor.getSegmentDependencies();
      for (i = 0; i < segmentClasses.length; i++) {
        this._register(segmentClasses[i]);}

      this.subscribeQueries(this.props);}


    /**
     * Unsubscribe all segments.
     */ }, { key: 'componentWillUnmount', value: 
    function componentWillUnmount() {
      console.log('[DATA] Unmounting', this);
      this.getReduxStore().unsubscribe(this);} }], [{ key: 'getSegmentDependencies', value: function getSegmentDependencies() {return [];} }]);return DataComponent;})(_react2['default'].Component);exports['default'] = DataComponent;module.exports = exports['default']; /**
                                                                                                                                                                                                                                                                              * @type {{[key: string]: Object}}
                                                                                                                                                                                                                                                                              */ /**
                                                                                                                                                                                                                                                                                  * @type {{[key: string]: Function}}
                                                                                                                                                                                                                                                                                  */
//# sourceMappingURL=DataComponent.js.map